unit MainUnit;

{
  This unit manages the other units in this sample, but doesn't really access
  the card. For how to acess the card, see the other form units.

  The exception is card detection. If you have multiple devices, you can do
  detection in roughly this manner. (If you only have one device, then
  its CardNum is simply 0, and detection is unnecessary.)
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    MainPages: TPageControl;
    SelSheet: TTabSheet;
    FunctionSheet: TTabSheet;
    Panel1: TPanel;
    Panel2: TPanel;
    CardNumLabel: TLabel;
    Panel3: TPanel;
    ForThisSampleLabel: TLabel;
    CardNameLabel: TLabel;
    CardDIDLabel: TLabel;
    CardRadios: TRadioGroup;
    SwitchBoxButton: TButton;
    PulseInButton: TButton;
    PulseOutButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure CardRadiosClick(Sender: TObject);
    procedure MainPagesChanging(Sender: TObject; var AllowChange: Boolean);
    procedure SwitchBoxButtonClick(Sender: TObject);
    procedure PulseInButtonClick(Sender: TObject);
    procedure PulseOutButtonClick(Sender: TObject);
  protected
    procedure FunctionButtonClick(FormClass: TFormClass; var FormRef);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOWDM,
  SampleGlobals, SwitchBoxUnit, PulseInUnit, PulseOutUnit;

procedure TMainForm.FormCreate(Sender: TObject);
var
  iCard, iSelCard: LongInt;
  NameSize: LongWord;
  CardStr: String;

begin
  LoadDefaultFont(Font);

  {
    This is the card detection. It fills in a card data record for each card.
    It calls GetNumCards to determine how many cards are present, then calls
    QueryCardInfo for each card to get its data.
  }
  iSelCard := -1;
  SetLength(CardData, GetNumCards);
  for iCard := 0 to High(CardData) do begin
    CardData[iCard].bForThisSample := False;

    NameSize := 256;
    SetLength(CardData[iCard].Name, NameSize);
    if QueryCardInfo(iCard, @CardData[iCard].DeviceID, nil, @NameSize, @CardData[iCard].Name[1]) = 0 then begin
      CardData[iCard].Name := '<Error ' + IntToStr(GetLastError) + ' Querying>';
      CardStr := CardData[iCard].Name;
    end
    else begin
      SetLength(CardData[iCard].Name, NameSize-1);

      CardData[iCard].bForThisSample := True;
      CardData[iCard].AdvancedA := [aInput]; // Only bit indices 0-7 ("Port A") support Digital Input Features
      CardData[iCard].AdvancedB := [aOutput]; // Only bit indices 8-15 ("Port B") support Digtial Output Features
      CardData[iCard].AdvancedCLo := [];
      CardData[iCard].AdvancedCHi := [];

      if iSelCard < 0 then iSelCard := iCard;

      CardStr := CardData[iCard].Name + ' [' + IntToHex(CardData[iCard].DeviceID, 4) + ']';
    end;

    CardStr := '[' + IntToStr(iCard) + '] ' + CardStr;
    CardRadios.Items.Add(CardStr);
  end;

  CardNum := iSelCard;
  CardRadios.ItemIndex := iSelCard;
  if iSelCard >= 0 then begin
    MainPages.ActivePage := FunctionSheet;
  end
  else begin
    CardRadiosClick(CardRadios); //It didn't get auto-called in this case, so call it manually.

    MainPages.ActivePage := SelSheet;
  end;
end;

procedure TMainForm.CardRadiosClick(Sender: TObject);
begin
  CardNum := CardRadios.ItemIndex;
  if CardNum < 0 then begin
    CardNumLabel.Caption := '';
    CardNameLabel.Caption := '<No Card Selected>';
    ForThisSampleLabel.Caption := '';
    CardDIDLabel.Caption := '';
  end
  else begin
    CardNumLabel.Caption := 'CardNum'#13#10 + IntToStr(CardNum);
    CardNameLabel.Caption := 'Name'#13#10 + CardData[CardNum].Name;
    CardDIDLabel.Caption := 'DeviceID'#13#10 + IntToHex(CardData[CardNum].DeviceID, 4);
  end;
end;

procedure TMainForm.MainPagesChanging(Sender: TObject; var AllowChange: Boolean);
begin
  if MainPages.ActivePage = SelSheet then begin
    if (CardNum < 0) or not CardData[CardNum].bForThisSample then
      AllowChange := False
    ;
  end
  else if MainPages.ActivePage = FunctionSheet then begin
    if SwitchBoxForm <> nil then SwitchBoxForm.Close;
    if PulseOutForm <> nil then PulseOutForm.Close;
    if PulseInForm <> nil then PulseInForm.Close;
  end;
end;

procedure TMainForm.FunctionButtonClick(FormClass: TFormClass; var FormRef);
begin
  if TForm(FormRef) = nil then begin
    Application.CreateForm(FormClass, FormRef);
  end;

  with TForm(FormRef) do begin
    Hide;
    Show;
    SetFocus;
  end;
end;

procedure TMainForm.SwitchBoxButtonClick(Sender: TObject);
begin
  FunctionButtonClick(TSwitchBoxForm, SwitchBoxForm);
end;

procedure TMainForm.PulseInButtonClick(Sender: TObject);
var
  Config: Byte;
  
begin
  //Set as input the groups with advanced inputs, leave the rest as they were.
  Config := RelInPortB(CardNum, 3);
  Config := Config or CONFIGDIR_PORTA;
  RelOutPortB(CardNum, 3, Config);
  FunctionButtonClick(TPulseInForm, PulseInForm);
end;

procedure TMainForm.PulseOutButtonClick(Sender: TObject);
var
  Config: Byte;
begin
  Config := RelInPortB(CardNum, 3);
  Config := Config and NOT CONFIGDIR_PORTB;
  RelOutPortB(CardNum, 3, Config);
  FunctionButtonClick(TPulseOutForm, PulseOutForm);
end;

end.
