unit SampleGlobals;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls;

const
  BaseClockHz = 125000000;
  
  CONFIGDIR_PORTA = $10;
  CONFIGDIR_PORTB = $02;
  CONFIGDIR_PORTC = $09;
  CONFIGDIR_PORTCLO = $01;
  CONFIGDIR_PORTCHI = $08;

type
  TAdvancedIO = set of (aInput, aOutput);
  TCardData = record
    Name: AnsiString;
    DeviceID: LongWord;
    bForThisSample: Boolean;
    AdvancedA, AdvancedB, AdvancedCLo, AdvancedCHi: TAdvancedIO;
  end;

  TDirection = (tdUp, tdDown, tdLeft, tdRight);
  TDirections = set of TDirection;

var
  CardNum: LongInt;
  CardData: array of TCardData;

function TimeIntervalToStr(dT: Extended): String;
function FreqToStr(Freq: Extended): String;

procedure LoadDefaultFont(IntoFont: TFont);
function CopyControlToIndex(NewOwner: TComponent; BaseControl: TControl; ToIndex: Integer; InDirection: TDirection): TControl;



implementation

function TimeIntervalToStr(dT: Extended): String;
begin
  if dT >= 1/1000000 then
    Result := FloatToStrF(dT*1000000, ffGeneral, 3, 0) + 's'
  else
    Result := FloatToStrF(dT*1000000000, ffGeneral, 3, 0) + 'ns'
  ;
end;

function FreqToStr(Freq: Extended): String;
begin
  if Freq >= 1000000 then
    Result := FloatToStrF(Freq/1000000, ffGeneral, 3, 0) + 'MHz'
  else if Freq >= 1000 then
    Result := FloatToStrF(Freq/1000, ffGeneral, 3, 0) + 'kHz'
  else
    Result := FloatToStrF(Freq, ffGeneral, 3, 0) + 'Hz'
  ;
end;



procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;

function CopyControlToIndex(NewOwner: TComponent; BaseControl: TControl; ToIndex: Integer; InDirection: TDirection): TControl;
var
  BR: TRect;
  X, Y, W, H: Integer;

begin
  Result := TControlClass(BaseControl.ClassType).Create(NewOwner);

  if Result is TButton then begin
    TButton(Result).OnClick := TButton(BaseControl).OnClick;
    TButton(Result).Caption := TButton(BaseControl).Caption;
  end
  else if Result is TLabel then begin
    TLabel(Result).AutoSize := TLabel(BaseControl).AutoSize;
    TLabel(Result).Alignment := TLabel(BaseControl).Alignment;
    TLabel(Result).Layout := TLabel(BaseControl).Layout;
    TLabel(Result).Caption := TLabel(BaseControl).Caption;
  end;

  Result.Tag := BaseControl.Tag + ToIndex;
  BR := BaseControl.BoundsRect;
  X := BR.Left;
  Y := BR.Top;
  W := BR.Right - BR.Left;
  H := BR.Bottom - BR.Top;
  case InDirection of
    tdUp: Y := Y - ToIndex * H;
    tdDown: Y := Y + ToIndex * H;
    tdLeft: X := X - ToIndex * W;
    tdRight: X := X + ToIndex * W;
  end;
  Result.SetBounds(X, Y, W, H);
  Result.Anchors := BaseControl.Anchors;
  Result.Align := BaseControl.Align;
  Result.Parent := BaseControl.Parent;
end;

end.
